----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:15:54 08/18/2016 
-- Design Name: 
-- Module Name:    SPI_readWriteReg - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity SPI_readWriteReg is
	Generic ( dataLength : integer := 16;   -- Lnge des Datenpaketes.
				 T_clk : integer := 10);		-- ns - Entspricht der Periodendauer von clk.
	Port (   clk : in  STD_LOGIC;		-- Takt.
				enable : in STD_LOGIC; 	-- Enable.
				rst : in STD_LOGIC;	-- Reset.
				selectController : in integer range 1 to 3;	-- Auswahl des Controllers zum Schreiben / Lesen. 
				readWriteBit_DriverIC : in STD_LOGIC; 	-- Lese- oder Schreibzugriff auf den Motor-Treiber IC
				readOrWriteIntReg : in STD_LOGIC; 	-- Lese- oder Schreibzugriff auf die im FPGA internen Datenregiser fr den Treiber-IC
																-- readOrWriteIntReg = '1' --> Set = Schreibe dataIn in das Register von selectController an die Stelle von address
																-- readOrWriteIntReg = '0' --> Read = Nimm die Daten aus dem Register von selectController an der Stelle von address
																--										        und gib sie an dataOut weiter. 
				regAddress : in  std_logic_vector(2 downto 0);	-- Adresse zum Schreiben (/Lesen).
				dataToIntReg : in std_logic_vector(11 downto 0);	-- Daten, die in das Register regAddress geschrieben werden sollen. 
				DATAfromDRIVER : in  STD_LOGIC;	-- Serielle Schnittstelle Output von Slave (dort SDATO), also Input fr Master. 
															-- Hier Output von Driver IC, Input fr FPGA.
				SCS : out  STD_LOGIC;	-- Chip-Select Signal zur Auswahl des Controll-ICs. 
				SCLK : out  STD_LOGIC;	-- Taktsignal zur Datenbertragung ber SPI.
				DATAtoDRIVER : out  STD_LOGIC;	-- Serielle Schnittstelle Input fr Slave (dort SDATI), also Output von Master. 
															-- Hier Input fr Driver IC, Output von FPGA.
				SPI_readData : out std_logic_vector(dataLength-5 downto 0); 	-- Gelesene Daten
				readWriteDone : out std_logic 	-- Lese- oder Schreibzugriff fr das entsprechende Register abgeschlossen. 
			); 
end SPI_readWriteReg;


------------------------------------------------------------------------------------
------------------------------------------------------------------------------------
-- Allgemeine Beschreibung der Funktion: 
-- 
-- Die entity SPI_readWriteReg hat mehrere Funktionen: es kann ein Register vom 
-- Treiber-IC gelesen werden, es kann ein Register des Treiber-ICs mit den Werten
-- aus dem entsprechenden internen Register im FPGA beschrieben werden oder es 
-- kann der Wert aus dem internen Register im FPGA berschrieben und der neue 
-- Wert gleich an den Treiber-IC weitergeben werden. 
-- 
-- Genauere Beschreibung: 
-- In dem Treiber-IC sind 8 Register mit jeweils 12 Bit vorhanden, ber die bestimmte
-- Parameter des Treiber-ICs eingestellt sind. Diese Parameter knnen ber SPI sowohl
-- gelesen, als auch geschrieben werden. 
-- Um die Parameter beim Starten des Treiber-ICs auf die gewnschten Werte zu setzen,
-- werden innerhalb des FPGAs ebenfalls 8 x 12 Bit Register verwendet, welche die 
-- entsprechenden Start-Werte speichern. Die Werte knnen, sobald dies gewnscht ist, 
-- aktualisiert und an den Treiber-IC geschickt werden. Damit werden die dort vorhandenen  
-- Werte berschrieben. 
-- In der Komponente contReg_readWrite_3controller sind alle Werte fr 3 Treiber-ICs
-- hinterlegt.
-- Mit serialData werden die Werte aus den Registern zusammen mit der Adresse und einem
-- Lese- oder Schreib-Bit auf die bertragung mit SPI vorbereitet. 
-- In SPI_interface erfolgt der eigentliche Lese- oder Schreib-Zugriff auf den Treiber-IC. 
-- 
-- Beschreibung der Ein- und Ausgangssignale: 
-- clk : Taktsignal 
-- enable : Enable-Signal. Startet den Prozess. 
-- rst : Reset-Signal
-- selectController : Auswahl des Controllers zum Schreiben / Lesen. Es sind derzeit 3 Kontroller hinterlegt. 
-- readWriteBit_DriverIC : Lese- oder Schreibzugriff auf den Motor-Treiber IC. Bei '0' wird auf den Kontroller geschrieben, 
-- 								bei '1' wird vom Controller gelesen!! 
-- readOrWriteIntReg : Lese- oder Schreibzugriff auf die im FPGA internen Datenregiser (fr den Treiber-IC)
-- 							readOrWriteIntReg = '1' --> Set = Schreibe dataIn in das Register von selectController an die Stelle von address
-- 							readOrWriteIntReg = '0' --> Read = Nimm die Daten aus dem Register von selectController an der Stelle von address
--																	        und gib sie an dataOut weiter. 
-- regAddress : Adresse zum Schreiben oder Lesen.
-- dataToIntReg : Daten, die in das Register regAddress geschrieben werden sollen. Diese Daten werden verwendet, wenn 
-- 					readOrWriteIntReg = '1' ist. In diesem Fall werden die Daten sowohl in das interne Register des FPGAs, 
-- 					als auch in das entsprechende Register des Treiber-ICs geschrieben! Bei einer Lese-Operation oder bei 
-- 					readOrWriteIntReg = '0' werden diese Daten ignoriert. 
-- DATAfromDRIVER : Serielle Schnittstelle Output von Slave (dort SDATO), also Input fr Master. Hier Output von Driver IC, Input fr FPGA.
-- 					  Die Schnittstelle dient zum lesen der Daten aus dem Treiber-IC. 
-- SCS : Chip-Select Signal zur Auswahl des Controll-ICs. 
-- SCLK : Taktsignal zur Datenbertragung ber SPI.
-- DATAtoDRIVER : Serielle Schnittstelle Input fr Slave (dort SDATI), also Output von Master. Hier Input fr Driver IC, Output von FPGA.
-- 					Die Schnittstelle dient zum schreiben von Daten auf den Treiber-IC. 
-- SPI_readData : Gelesene Daten aus dem Treiber-IC
-- readWriteDone : Lese- oder Schreibzugriff fr das entsprechende Register abgeschlossen. 
------------------------------------------------------------------------------------
------------------------------------------------------------------------------------


architecture Behavioral of SPI_readWriteReg is
	
	signal selectController_int : integer range 1 to 3;
	signal regAddress_int : std_logic_vector(2 downto 0); 
	signal readOrWriteIntReg_int : std_logic; 
	
	signal contrReg_enable : std_logic; 
	signal serialData_enable : std_logic; 
	signal SPI_interface_enable : std_logic; 
	signal regData : std_logic_vector(11 downto 0); 
	signal dataToIntReg_int : std_logic_vector(11 downto 0); 
	signal SPI_writeData : std_logic_vector(15 downto 0); 
	
	signal readWriteBit_DriverIC_int : std_logic; 
	
	signal dataAvailable : std_logic; 
	signal SCS_int : std_logic; 
		
	signal dataAvailable_shiftReg : std_logic_vector(1 downto 0); 	
	signal serialData_shiftReg : std_logic_vector(1 downto 0); 
	signal SPI_interface_SCS_shiftReg : std_logic_vector(1 downto 0); 
	
	
	
	component contReg_readWrite_3controller
	port(
         clk : in  std_logic;
         enable : in  std_logic;
         readOrWriteIntReg : in  std_logic;
         rst : in  std_logic;
         address : in  std_logic_vector(2 downto 0);
         selectController : in  integer range 1 to 3;
         dataIn : in  std_logic_vector(11 downto 0);
         dataOut : out  std_logic_vector(11 downto 0);
         dataAvailable : out  std_logic
        );
	end component;
	
	
	component serialData
		Port(
			clk : in std_logic; 
			enable : in std_logic;
			readWriteBit : in std_logic; 	
			address : in std_logic_vector(2 downto 0); 
			writeData : in std_logic_vector(11 downto 0); 
			dataOut : out std_logic_vector(15 downto 0) 
		); 
	end component; 
	
		
	component SPI_interface
		 Generic ( dataLength : integer;   	-- Lnge des Datenpaketes
					  T_clk : integer);			-- ns - Entspricht der Periodendauer von clk.
		 Port ( clk : in  STD_LOGIC;	
				  enable : in  STD_LOGIC;	
				  writeData : in std_logic_vector(dataLength-1 downto 0); 
				  readData : out std_logic_vector(dataLength-5 downto 0); 
				  rst : in  STD_LOGIC;		
				  SCS : out  STD_LOGIC;		
				  SCLK : out  STD_LOGIC;	
				  DATAtoDRIVER : out  STD_LOGIC;	
				  DATAfromDRIVER : in  STD_LOGIC
		);
	end component; 
	
	
	
begin

	
	contReg_readWrite_3controller_inst : contReg_readWrite_3controller
		Port map (
			clk => clk,
			enable => contrReg_enable,
			readOrWriteIntReg => readOrWriteIntReg_int,
			rst => rst,
			address => regAddress_int,
			selectController => selectController_int,
			dataIn => dataToIntReg_int,
			dataOut => regData,
			dataAvailable => dataAvailable);
			

	
	serialData_inst : serialData
		Port map ( 
			clk => clk,
			enable => serialData_enable,
			readWriteBit => readWriteBit_DriverIC_int,
			address => regAddress_int,
			writeData => regData, 
			dataOut => SPI_writeData); 	
			

		
	SPI_interface_inst : SPI_interface 
		 generic map ( dataLength => dataLength,   -- Lnge des Datenpaketes
							T_clk => T_clk)		-- ns - Entspricht der Periodendauer von clk.
		 Port map ( 
					clk => clk,
					enable => SPI_interface_enable,
					writeData => SPI_writeData, 
					readData => SPI_readData, 
					rst => rst,		
					SCS => SCS_int,		
					SCLK => SCLK,	
					DATAtoDRIVER => DATAtoDRIVER,
					DATAfromDRIVER => DATAfromDRIVER);	-- Es wird derzeit nur geschrieben, nicht gelesen!! 
	
	
	SCS <= SCS_int; 
	
	
	-----------------------------------------
	-- In contReg_readWrite_3controller liegen die FPGA-internen Register fr den Treiber-IC. 
	-- Die Werte knnen gelesen oder berschrieben werden. In beiden Fllen werden die (u.U. neuen)
	-- Werte an den Ausgang bergeben. 
	-- Im Anschluss werden diese Daten in serialData auf die bertragung mit SPI vorbereitet. 
	-- Hierfr wird ein Schreib- (oder Lese-)Bit sowie die Adresse angehngt. 
	-- Im letzten Schritt werden die Daten in SPI_interface an den Treiber-IC bertragen oder 
	-- von diesem gelesen. 
	-- 
	-- In dem folgenden Prozess werden diese drei Schritte nacheinander ausgefhrt. Um die Reihenfolge
	-- und das Timing einzuhalten, werden mehrere Schieberegister verwendet. Ist ein Schritt ausgefhrt, 
	-- wird ein entsprechendes Signal in dem Schieberegister gesetzt. Nachdem dieses Signal das Ende des 
	-- Schieberegisters erreicht hat, wird das enable-Signal fr den nchsten Schritt gesetzt und dieser
	-- ausgefhrt. Ist der letzte Schritt durchgefhrt worden, wird dies durch readWriteDone angezeigt. 
	-- 
	-- Bei einem Leseprozess muss contReg_readWrite_3controller nicht aufgerufen werden. Hier wird dieser
	-- Teil trotzdem aufgerufen, damit der Ablauf gleich dem eines Schreibprozesses ist. Die Daten, welche
	-- aus contReg_readWrite_3controller abgerufen werden, werden jedoch in serialData bei einem Leseprozess
	-- verworfen. Die Umsetzung ist auf diesem Weg jedoch deutlich erleichtert. 
	
	process 
	begin
		wait until rising_edge(clk); 		

		contrReg_enable <= '0'; 
		serialData_enable <= '0'; 
		SPI_interface_enable <= '0'; 
		readWriteDone <= '0'; 
				
		if enable = '1' then	
			contrReg_enable <= '1'; 
			
			selectController_int <= selectController; 
			regAddress_int <= regAddress; 
			dataToIntReg_int <= dataToIntReg; 
			
			readWriteBit_DriverIC_int <= readWriteBit_DriverIC; 
			
			if readWriteBit_DriverIC = '1' then  	-- 0 fr Write; 1 fr Read.
				readOrWriteIntReg_int <= '0';			-- '0' --> Read; '1' --> Set;   Hier soll aus dem internen Register im FPGA gelesen werden, 
																-- wobei der Inhalt verworfen wird, da im Anschluss die Daten aus dem Treiber-IC gelesen
																-- werden. Dies dient nur dazu, dass der Ablauf contReg - serialData - SPI_interface im Anschluss
																-- gleich bleiben kann und dem vom lesen oder schreiben des internen Registers entspricht. 
			else 
				readOrWriteIntReg_int <= readOrWriteIntReg; 
			end if; 
		end if; 
		
		
		dataAvailable_shiftReg <= dataAvailable_shiftReg(dataAvailable_shiftReg'left-1 downto 0) & dataAvailable; 
		
		if dataAvailable_shiftReg = "10" then 
			serialData_enable <= '1'; 
		end if; 
		
		
		serialData_shiftReg <= serialData_shiftReg(serialData_shiftReg'left-1 downto 0) & serialData_enable; 
		
		if serialData_shiftReg = "10" then 
			SPI_interface_enable <= '1'; 
		end if; 
		
		
		SPI_interface_SCS_shiftReg <= SPI_interface_SCS_shiftReg(SPI_interface_SCS_shiftReg'left-1 downto 0) & SCS_int; 

		if SPI_interface_SCS_shiftReg = "10" then 
			readWriteDone <= '1'; 
		end if; 
			
			
		if rst = '1' then 
			contrReg_enable <= '0'; 
			serialData_enable <= '0'; 
			SPI_interface_enable <= '0'; 
			
			dataAvailable_shiftReg <= (others => '0'); 
			serialData_shiftReg <= (others => '0'); 
			SPI_interface_SCS_shiftReg <= (others => '0'); 
			
			readWriteDone <= '0'; 
		end if; 
	end process; 

	
	
	
	
	
end Behavioral;

