----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:45:21 08/18/2016 
-- Design Name: 
-- Module Name:    contReg_readWrite_3controller - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity contReg_readWrite_3controller is
	Port(
		clk : in std_logic; 
		enable : in std_logic; 		-- Enable zur bernahme einer neuen Adresse und damit auch neuer Daten, die geschrieben werden soll. 
		readOrWriteIntReg : in std_logic; 	-- Legt fest, ob die Daten an der Adresse aus dem Register hier gelesen werden sollen, oder ob 
												-- der Wert in diesem Register hier berschrieben werden soll. 
												-- readOrWriteIntReg = '1' --> Set = Schreibe dataIn in das Register von selectController an die Stelle von address
												-- readOrWriteIntReg = '0' --> Read = Nimm die Daten aus dem Register von selectController an der Stelle von address
												--										        und gib sie an dataOut weiter. 
		rst : in std_logic; 
		address : in std_logic_vector(2 downto 0); 		-- Adresse zum Schreiben (/Lesen).
		selectController : in integer range 1 to 3; 		-- Auswahl des Controllers (es werden maximal 3 Controller verwendet!)
		dataIn : in std_logic_vector(11 downto 0);
		dataOut : out std_logic_vector(11 downto 0);		
		dataAvailable : out std_logic			-- wird gesetzt, wenn die Daten geschrieben wurden oder am Ausgang bereit liegen. 
	);
end contReg_readWrite_3controller;


----------------------------------------------------------------------
-- In controlRegisters sind die Kontroll-Register des Controller-ICs hinterlegt und 
-- mit Werten gefllt. Diese Werte werden durch die bergebene Adresse abgefragt 
-- und an den Ausgang bergeben. 
-- Zustzlich ist es hier mglich, einzelne Werte zu berschreiben. 
-- 
-- Es werden insgesamt drei Motoren verwendet, wodurch auch drei Controller-ICs 
-- bentigt werden. Um jedem Controller-IC andere Daten bergeben zu knnen, 
-- werden alle Kontroll-Register fr die drei ICs hier separat aufgefhrt. 
-- Die Auswahl des entsprechenden ICs erfolgt ber selectController. 
----------------------------------------------------------------------


architecture Behavioral of contReg_readWrite_3controller is


--	type registerName is (CTRL, TORQUE, OFF, BLANK, DECAY, STALL, DRIVE, STATUS); 
--	signal reg : registerName; 

	signal reg : natural range 0 to 7; 
		
	type controlRegister is array (0 to 7) of std_logic_vector(11 downto 0); 
	signal controlRegisterController1 : controlRegister := (	"11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1",		-- CTRL
																				"0" & 		"001" & 	"00101011",		-- TORQUE
																				"000" & 	"0" & 	"00001011",		-- OFF
																				"000" & 	"0" & 	"00110001",		-- BLANK
																				"0" & 		"101" & 		"00000100",		-- DECAY
																				"11" &	"11" & 	"01000000",		-- STALL
																				"01" &		"01" & 	"01" &	"01" & 	"01" &	"00",		-- DRIVE
																				"0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0");		-- STATUS
	signal controlRegisterController2 : controlRegister := (	"11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1",		-- CTRL
																				"0" & 		"001" & 	"00101011",		-- TORQUE
																				"000" & 	"0" & 	"00001011",		-- OFF
																				"000" & 	"0" & 	"00110001",		-- BLANK
																				"0" & 		"101" & 		"00000100",		-- DECAY
																				"11" &	"11" & 	"01000000",		-- STALL
																				"01" &		"01" & 	"01" &	"01" & 	"01" &	"00",		-- DRIVE
																				"0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0");		-- STATUS
	signal controlRegisterController3 : controlRegister := (	"11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1",		-- CTRL
																				"0" & 		"001" & 	"00101011",		-- TORQUE
																				"000" & 	"0" & 	"00001011",		-- OFF
																				"000" & 	"0" & 	"00110001",		-- BLANK
																				"0" & 		"101" & 		"00000100",		-- DECAY
																				"11" &	"11" & 	"01000000",		-- STALL
																				"01" &		"01" & 	"01" &	"01" & 	"01" &	"00",		-- DRIVE
																				"0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0");		-- STATUS
	
	signal startReadOrWrite : std_logic; 
	signal readOrWrite_int : std_logic; 
	signal dataIn_int : std_logic_vector(11 downto 0);
	
	
begin


	process
	begin
		wait until rising_edge(clk); 
		if enable = '1' then 
			
			startReadOrWrite <= '1'; 
			readOrWrite_int <= readOrWriteIntReg; 
			dataIn_int <= dataIn; 
		
			CASE address IS
				WHEN "000" => --	Hex Adresse 00
					reg <= 0;	-- CTRL
					
				WHEN "001" => --	Hex Adresse 01
					reg <= 1;	-- TORQUE
					
				WHEN "010" => --	Hex Adresse 02
					reg <= 2;	-- OFF
					
				WHEN "011" => --	Hex Adresse 03
					reg <= 3;	-- BLANK
				
				WHEN "100" => --	Hex Adresse 04
					reg <= 4;	-- DECAY
				
				WHEN "101" => --	Hex Adresse 05
					reg <= 5;	-- STALL
				
				WHEN "110" => --	Hex Adresse 06
					reg <= 6;	-- DRIVE
				
				WHEN "111" => --	Hex Adresse 07
					reg <= 7;	-- STATUS
					
				WHEN others => 
					null; 
				
			END CASE; 
		
		else	
			startReadOrWrite <= '0'; 
		end if; 
		
		if rst = '1' then 
			startReadOrWrite <= '0'; 
		end if; 
	end process; 
	
	
	
	
	process
	begin
		wait until rising_edge(clk); 
		dataAvailable <= '0'; 
		
		if startReadOrWrite = '1' then 
			if readOrWrite_int = '1' then 
				if selectController = 1 then 
					controlRegisterController1(reg) <= dataIn_int; 
				elsif selectController = 2 then 
					controlRegisterController2(reg) <= dataIn_int; 
				elsif selectController = 3 then 
					controlRegisterController3(reg) <= dataIn_int; 
				end if; 
				dataOut <= dataIn_int; 
			else
				if selectController = 1 then 
					dataOut <= controlRegisterController1(reg); 
				elsif selectController = 2 then 
					dataOut <= controlRegisterController2(reg); 
				elsif selectController = 3 then 
					dataOut <= controlRegisterController3(reg); 
				end if; 
			end if; 
			
			dataAvailable <= '1'; 

		end if; 
		
		
		if rst = '1' then 
----------------------------------------------------------------
----------------------------------------------------------------
--------------------- AB HIER CONTROLLER 1 ---------------------
----------------------------------------------------------------
----------------------------------------------------------------
			
		-- For Stepper Motor Controller IC DRV8711 (Texas Instruments) 

-----------------------------------------------------------------------------------------
--		CTRL
		
				-- BIT	NAME		SIZE	R/W	DEFAULT	DESCRIPTION
				-- 0 		ENBL		1		R/W	0			0 : Disable Motor
				-- 												1 : Enable Motor
				-- 1 		RDIR		1		R/W	0			0 : Direction set by DIR pin
				-- 												1 : Direction set by inverse of DIR pin
				-- 2 		RSTEP 	1 		W 		0 			0: No action
				-- 												1: Indexer will advance one step; automatically cleared after write
				-- 6-3 	MODE 		4 		R/W 	0010 		0000: Full-step, 71% current
				--													0001: Half step
				--													0010: 1/4 step
				--													0011: 1/8 step
				--													0100: 1/16 step 
				-- 												0101: 1/32 step
				--													0110: 1/64 step
				--													0111: 1/128 step
				--													1000: 1/256 step
				-- 												1001 - 1111: Reserved
				-- 7 		EXSTALL 	1 		R/W 	0 			0: Internal stall detect 
				-- 												1: External stall detect
				-- 9-8 	ISGAIN 	2		R/W 	00 		ISENSE amplifier gain set
				-- 												00: Gain of 5
				-- 												01: Gain of 10
				-- 												10: Gain of 20
				-- 												11: Gain of 40
				-- 11-10 DTIME 	2 		R/W 	11 		Dead time set
				-- 												00: 400 ns dead time
				-- 												01: 450 ns dead time
				-- 												10: 650 ns dead time
				-- 												11: 850 ns dead time
				
				-- DEFAULT: 
				--				DTIME		ISGAIN	EXSTALL	MODE		RSTEP		RDIR		ENBL
--					data <= 	"11" & 	"00" & 	"0" & 	"0010" & "0" & 	"0" & 	"0"; 

				-- TEST: 
--					data <= 	"11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1"; 	-- 
		
		controlRegisterController1(0) <= "11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------		
--		TORQUE
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				--	7-0 	TORQUE 	8 		R/W 	0xFFh 	Sets full-scale output current for both H-bridges
				--	10-8 	SMPLTH 	3 		R/W 	001 		Back EMF sample threshold
				--													000: 50 s
				--													001: 100 s
				--													010: 200 s
				--													011: 300 s
				--													100: 400 s
				--													101: 600 s
				--													110: 800 s
				--													111: 1000 s
				--	11		Reserved	1 		- 		- 			Reserved
				
				-- DEFAULT: 
				-- 		  Reserved	SMPLTH	TORQUE
--					data <= "0" & 		"001" & 	"11111111";
				
				-- TEST: 
--					data <= "0" & 		"001" & 	"00101011";		--

		controlRegisterController1(1) <= "0" & 		"001" & 	"00101011";
-----------------------------------------------------------------------------------------	


-----------------------------------------------------------------------------------------
--		OFF
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	TOFF 		8 		R/W 	0x30h 	Sets fixed off time, in increments of 500 ns
				-- 												0x00h: 500 ns
				-- 												0xFFh: 128 s
				-- 8 		PWMMODE 	1 		R/W 	0 			0: Use internal indexer 
				-- 												1: Bypass indexer, use xINx inputs to control outputs
				--11-9	Reserved	3 		- 		- 			Reserved
				
				-- DEFAULT: 
				-- 		  Reserved	PWMMODE	TOFF
--					data <= "000" & 	"0" & 	"00110000";
				
				-- TEST: 
--					data <= "000" & 	"0" & 	"00001011";		-- 
				
		controlRegisterController1(2) <= "000" & 	"0" & 	"00001011";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--		BLANK
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				--	7-0 	TBLANK 	8 		R/W 	0x80h 	Sets current trip blanking time, in increments of 20 ns
				-- 												(bei den jetzt aufgefhrten Zeiten scheint ein Fehler im Datenblatt zu sein!)
				--													0x00h: 1 s
				-- 												
				-- 												0x32h: 1 s	 
				-- 												0x33h: 1.02 s
				-- 												
				-- 												0xFEh: 5.10 s
				-- 												0xFFh: 5.12 s
				-- 												Also sets minimum on-time of PWM
				-- 8 		ABT 		1 		R/W 	0 			0: Disable adaptive blanking time 1: Enable adaptive blanking time
				-- 11-9 	Reserved	3 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	ABT		TBLANK
--					data <= "000" & 	"0" & 	"10000000";
				
				-- TEST: 
--					data <= "000" & 	"0" & 	"00110001";		-- 
				
		controlRegisterController1(3) <= "000" & 	"0" & 	"00110001";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			DECAY
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	TDECAY 	8 		R/W 	0x10h 	Sets mixed decay transition time, in increments of 500 ns
				-- 10-8 	DECMOD 	3 		R/W 	001 		000: Force slow decay at all times
				--													001: Slow decay for increasing current, mixed decay for
				--														  decreasing current (indexer mode only)
				--													010: Force fast decay at all times
				-- 												011: Use mixed decay at all times
				--													100: Slow decay for increasing current, auto mixed decay for
				--														  decreasing current (indexer mode only)
				--													101: Use auto mixed decay at all times
				--													110 - 111: Reserved
				-- 11 	Reserved	1 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	DECMOD		TDECAY
--					data <= "0" & 		"001" & 		"00010000";
				
				-- TEST: 
--					data <= "0" & 		"101" & 		"00000100";		-- 
				
		controlRegisterController1(4) <= "0" & 		"101" & 		"00000100";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------	
--			STALL 
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	SDTHR 	8 		R/W 	0x40h 	Sets stall detect threshold The correct setting needs to be determined experimentally
				-- 9-8 	SDCNT 	2 		R/W 	00 		00: STALLn asserted on first step with back EMF below SDTHR
				-- 												01: STALLn asserted after 2 steps 
				-- 												10: STALLn asserted after 4 steps
				--													11: STALLn asserted after 8 steps
				--11-10 	VDIV 		2 		R/W 	00 		00: Back EMF is divided by 32
				-- 												01: Back EMF is divided by 16 
				--													10: Back EMF is divided by 8
				--													11: Back EMF is divided by 4
				
				-- DEFAULT: 
				--			  VDIV	SDCNT		SDTHR
--					data <= "00" &	"00" & 	"01000000";
				
				-- TEST: 
--					data <= "11" &	"11" & 	"01000000";		-- SDTHR = 64 
				
		controlRegisterController1(5) <= "11" &	"11" & 	"01000000";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			DRIVE
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 1-0 	OCPTH 	2 		R/W 	00 		OCP threshold
				--													00: 250 mV
				-- 												01: 500 mV
				--													10: 750 mV
				--													11: 1000 mV
				-- 3-2 	OCPDEG 	2 		R/W 	01 		OCP deglitch time
				--													00: 1 s
				-- 												01: 2 s
				--													10: 4 s
				--													11: 8 s
				-- 5-4 	TDRIVEN 	2 		R/W 	01 		Low-side gate drive time
				--													00: 250 ns
				-- 												01: 500 ns
				--													10: 1 s
				--													11: 2 s
				-- 7-6 	TDRIVEP 	2 		R/W 	01 		High-side gate drive time
				--													00: 250 ns
				-- 												01: 500 ns
				--													10: 1 s
				--													11: 2 s
				-- 9-8 	IDRIVEN 	2 		R/W 	00			Low-side gate drive peak current
				--													00: 100 mA peak (sink)
				-- 												01: 200 mA peak (sink)
				--													10: 300 mA peak (sink)
				--													11: 400 mA peak (sink)
				-- 11-10	IDRIVEP 	2 		R/W 	00			High-side gate drive peak current
				--													00: 50 mA peak (source)
				-- 												01: 100 mA peak (source)
				--													10: 150 mA peak (source)
				--													11: 200 mA peak (source)
				
				-- DEFAULT: 
				--			  IDRIVEP	IDRIVEN	TDRIVEP	TDRIVEN	OCPDEG	OCPTH
--					data <= "00" &		"00" & 	"01" &	"01" & 	"01" &	"00";
				
				-- TEST: 
--					data <= "01" &		"01" & 	"01" &	"01" & 	"01" &	"00";
				
		controlRegisterController1(6) <= "01" &		"01" & 	"01" &	"01" & 	"01" &	"00";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			STATUS
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 0 		OTS 		1 		R 		0 			0: Normal operation
				-- 												1: Device has entered overtemperature shutdown
				--													OTS bit will clear once temperature has fallen to safe levels
				-- 1 		AOCP 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel A overcurrent shutdown
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 2 		BOCP 		1 		R/W 	0			0: Normal operation
				-- 												1: Channel B overcurrent shutdown
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 3 		APDF 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel A predriver fault
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 4 		BPDF 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel B predriver fault
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 5 		UVLO 		1 		R 		0 			0: Normal operation
				-- 												1: Undervoltage lockout
				--													UVLO bit will clear after VM has increased over VUVLO
				-- 6 		STD 		1 		R 		0 			0: Normal operation 
				-- 												1: Stall detected
				-- 7 		STDLAT 	1 		R/W 	0 			0: Normal operation
				-- 												1: Latched stall detect
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 11-8 	Reserved	4 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	STDLAT	STD	UVLO	BPDF	APDF	BOCP	AOCP	OTS
--					data <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
				
				-- TEST: 
--					data <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
				
		controlRegisterController1(7) <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
-----------------------------------------------------------------------------------------

----------------------------------------------------------------
----------------------------------------------------------------
--------------------- BIS HIER CONTROLLER 1 --------------------
----------------------------------------------------------------
----------------------------------------------------------------



----------------------------------------------------------------
----------------------------------------------------------------
--------------------- AB HIER CONTROLLER 2 ---------------------
----------------------------------------------------------------
----------------------------------------------------------------
			
		-- For Stepper Motor Controller IC DRV8711 (Texas Instruments) 

-----------------------------------------------------------------------------------------
--		CTRL
		
				-- BIT	NAME		SIZE	R/W	DEFAULT	DESCRIPTION
				-- 0 		ENBL		1		R/W	0			0 : Disable Motor
				-- 												1 : Enable Motor
				-- 1 		RDIR		1		R/W	0			0 : Direction set by DIR pin
				-- 												1 : Direction set by inverse of DIR pin
				-- 2 		RSTEP 	1 		W 		0 			0: No action
				-- 												1: Indexer will advance one step; automatically cleared after write
				-- 6-3 	MODE 		4 		R/W 	0010 		0000: Full-step, 71% current
				--													0001: Half step
				--													0010: 1/4 step
				--													0011: 1/8 step
				--													0100: 1/16 step 
				-- 												0101: 1/32 step
				--													0110: 1/64 step
				--													0111: 1/128 step
				--													1000: 1/256 step
				-- 												1001 - 1111: Reserved
				-- 7 		EXSTALL 	1 		R/W 	0 			0: Internal stall detect 
				-- 												1: External stall detect
				-- 9-8 	ISGAIN 	2		R/W 	00 		ISENSE amplifier gain set
				-- 												00: Gain of 5
				-- 												01: Gain of 10
				-- 												10: Gain of 20
				-- 												11: Gain of 40
				-- 11-10 DTIME 	2 		R/W 	11 		Dead time set
				-- 												00: 400 ns dead time
				-- 												01: 450 ns dead time
				-- 												10: 650 ns dead time
				-- 												11: 850 ns dead time
				
				-- DEFAULT: 
				--				DTIME		ISGAIN	EXSTALL	MODE		RSTEP		RDIR		ENBL
--					data <= 	"11" & 	"00" & 	"0" & 	"0010" & "0" & 	"0" & 	"0"; 

				-- TEST: 
--					data <= 	"11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1"; 	-- 
		
		controlRegisterController2(0) <= "11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------		
--		TORQUE
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				--	7-0 	TORQUE 	8 		R/W 	0xFFh 	Sets full-scale output current for both H-bridges
				--	10-8 	SMPLTH 	3 		R/W 	001 		Back EMF sample threshold
				--													000: 50 s
				--													001: 100 s
				--													010: 200 s
				--													011: 300 s
				--													100: 400 s
				--													101: 600 s
				--													110: 800 s
				--													111: 1000 s
				--	11		Reserved	1 		- 		- 			Reserved
				
				-- DEFAULT: 
				-- 		  Reserved	SMPLTH	TORQUE
--					data <= "0" & 		"001" & 	"11111111";
				
				-- TEST: 
--					data <= "0" & 		"001" & 	"00101011";		--

		controlRegisterController2(1) <= "0" & 		"001" & 	"00101011";
-----------------------------------------------------------------------------------------	


-----------------------------------------------------------------------------------------
--		OFF
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	TOFF 		8 		R/W 	0x30h 	Sets fixed off time, in increments of 500 ns
				-- 												0x00h: 500 ns
				-- 												0xFFh: 128 s
				-- 8 		PWMMODE 	1 		R/W 	0 			0: Use internal indexer 
				-- 												1: Bypass indexer, use xINx inputs to control outputs
				--11-9	Reserved	3 		- 		- 			Reserved
				
				-- DEFAULT: 
				-- 		  Reserved	PWMMODE	TOFF
--					data <= "000" & 	"0" & 	"00110000";
				
				-- TEST: 
--					data <= "000" & 	"0" & 	"00001011";		-- 
				
		controlRegisterController2(2) <= "000" & 	"0" & 	"00001011";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--		BLANK
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				--	7-0 	TBLANK 	8 		R/W 	0x80h 	Sets current trip blanking time, in increments of 20 ns
				-- 												(bei den jetzt aufgefhrten Zeiten scheint ein Fehler im Datenblatt zu sein!)
				--													0x00h: 1 s
				-- 												
				-- 												0x32h: 1 s	 
				-- 												0x33h: 1.02 s
				-- 												
				-- 												0xFEh: 5.10 s
				-- 												0xFFh: 5.12 s
				-- 												Also sets minimum on-time of PWM
				-- 8 		ABT 		1 		R/W 	0 			0: Disable adaptive blanking time 1: Enable adaptive blanking time
				-- 11-9 	Reserved	3 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	ABT		TBLANK
--					data <= "000" & 	"0" & 	"10000000";
				
				-- TEST: 
--					data <= "000" & 	"0" & 	"00110001";		-- 
				
		controlRegisterController2(3) <= "000" & 	"0" & 	"00110001";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			DECAY
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	TDECAY 	8 		R/W 	0x10h 	Sets mixed decay transition time, in increments of 500 ns
				-- 10-8 	DECMOD 	3 		R/W 	001 		000: Force slow decay at all times
				--													001: Slow decay for increasing current, mixed decay for
				--														  decreasing current (indexer mode only)
				--													010: Force fast decay at all times
				-- 												011: Use mixed decay at all times
				--													100: Slow decay for increasing current, auto mixed decay for
				--														  decreasing current (indexer mode only)
				--													101: Use auto mixed decay at all times
				--													110 - 111: Reserved
				-- 11 	Reserved	1 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	DECMOD		TDECAY
--					data <= "0" & 		"001" & 		"00010000";
				
				-- TEST: 
--					data <= "0" & 		"101" & 		"00000100";		-- 
				
		controlRegisterController2(4) <= "0" & 		"101" & 		"00000100";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------	
--			STALL 
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	SDTHR 	8 		R/W 	0x40h 	Sets stall detect threshold The correct setting needs to be determined experimentally
				-- 9-8 	SDCNT 	2 		R/W 	00 		00: STALLn asserted on first step with back EMF below SDTHR
				-- 												01: STALLn asserted after 2 steps 
				-- 												10: STALLn asserted after 4 steps
				--													11: STALLn asserted after 8 steps
				--11-10 	VDIV 		2 		R/W 	00 		00: Back EMF is divided by 32
				-- 												01: Back EMF is divided by 16 
				--													10: Back EMF is divided by 8
				--													11: Back EMF is divided by 4
				
				-- DEFAULT: 
				--			  VDIV	SDCNT		SDTHR
--					data <= "00" &	"00" & 	"01000000";
				
				-- TEST: 
--					data <= "11" &	"11" & 	"01000000";		-- SDTHR = 64 
				
		controlRegisterController2(5) <= "11" &	"11" & 	"01000000";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			DRIVE
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 1-0 	OCPTH 	2 		R/W 	00 		OCP threshold
				--													00: 250 mV
				-- 												01: 500 mV
				--													10: 750 mV
				--													11: 1000 mV
				-- 3-2 	OCPDEG 	2 		R/W 	01 		OCP deglitch time
				--													00: 1 s
				-- 												01: 2 s
				--													10: 4 s
				--													11: 8 s
				-- 5-4 	TDRIVEN 	2 		R/W 	01 		Low-side gate drive time
				--													00: 250 ns
				-- 												01: 500 ns
				--													10: 1 s
				--													11: 2 s
				-- 7-6 	TDRIVEP 	2 		R/W 	01 		High-side gate drive time
				--													00: 250 ns
				-- 												01: 500 ns
				--													10: 1 s
				--													11: 2 s
				-- 9-8 	IDRIVEN 	2 		R/W 	00			Low-side gate drive peak current
				--													00: 100 mA peak (sink)
				-- 												01: 200 mA peak (sink)
				--													10: 300 mA peak (sink)
				--													11: 400 mA peak (sink)
				-- 11-10	IDRIVEP 	2 		R/W 	00			High-side gate drive peak current
				--													00: 50 mA peak (source)
				-- 												01: 100 mA peak (source)
				--													10: 150 mA peak (source)
				--													11: 200 mA peak (source)
				
				-- DEFAULT: 
				--			  IDRIVEP	IDRIVEN	TDRIVEP	TDRIVEN	OCPDEG	OCPTH
--					data <= "00" &		"00" & 	"01" &	"01" & 	"01" &	"00";
				
				-- TEST: 
--					data <= "01" &		"01" & 	"01" &	"01" & 	"01" &	"00";
				
		controlRegisterController2(6) <= "01" &		"01" & 	"01" &	"01" & 	"01" &	"00";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			STATUS
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 0 		OTS 		1 		R 		0 			0: Normal operation
				-- 												1: Device has entered overtemperature shutdown
				--													OTS bit will clear once temperature has fallen to safe levels
				-- 1 		AOCP 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel A overcurrent shutdown
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 2 		BOCP 		1 		R/W 	0			0: Normal operation
				-- 												1: Channel B overcurrent shutdown
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 3 		APDF 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel A predriver fault
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 4 		BPDF 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel B predriver fault
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 5 		UVLO 		1 		R 		0 			0: Normal operation
				-- 												1: Undervoltage lockout
				--													UVLO bit will clear after VM has increased over VUVLO
				-- 6 		STD 		1 		R 		0 			0: Normal operation 
				-- 												1: Stall detected
				-- 7 		STDLAT 	1 		R/W 	0 			0: Normal operation
				-- 												1: Latched stall detect
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 11-8 	Reserved	4 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	STDLAT	STD	UVLO	BPDF	APDF	BOCP	AOCP	OTS
--					data <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
				
				-- TEST: 
--					data <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
				
		controlRegisterController2(7) <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
-----------------------------------------------------------------------------------------

----------------------------------------------------------------
----------------------------------------------------------------
--------------------- BIS HIER CONTROLLER 2 --------------------
----------------------------------------------------------------
----------------------------------------------------------------



----------------------------------------------------------------
----------------------------------------------------------------
--------------------- AB HIER CONTROLLER 3 ---------------------
----------------------------------------------------------------
----------------------------------------------------------------
			
		-- For Stepper Motor Controller IC DRV8711 (Texas Instruments) 

-----------------------------------------------------------------------------------------
--		CTRL
		
				-- BIT	NAME		SIZE	R/W	DEFAULT	DESCRIPTION
				-- 0 		ENBL		1		R/W	0			0 : Disable Motor
				-- 												1 : Enable Motor
				-- 1 		RDIR		1		R/W	0			0 : Direction set by DIR pin
				-- 												1 : Direction set by inverse of DIR pin
				-- 2 		RSTEP 	1 		W 		0 			0: No action
				-- 												1: Indexer will advance one step; automatically cleared after write
				-- 6-3 	MODE 		4 		R/W 	0010 		0000: Full-step, 71% current
				--													0001: Half step
				--													0010: 1/4 step
				--													0011: 1/8 step
				--													0100: 1/16 step 
				-- 												0101: 1/32 step
				--													0110: 1/64 step
				--													0111: 1/128 step
				--													1000: 1/256 step
				-- 												1001 - 1111: Reserved
				-- 7 		EXSTALL 	1 		R/W 	0 			0: Internal stall detect 
				-- 												1: External stall detect
				-- 9-8 	ISGAIN 	2		R/W 	00 		ISENSE amplifier gain set
				-- 												00: Gain of 5
				-- 												01: Gain of 10
				-- 												10: Gain of 20
				-- 												11: Gain of 40
				-- 11-10 DTIME 	2 		R/W 	11 		Dead time set
				-- 												00: 400 ns dead time
				-- 												01: 450 ns dead time
				-- 												10: 650 ns dead time
				-- 												11: 850 ns dead time
				
				-- DEFAULT: 
				--				DTIME		ISGAIN	EXSTALL	MODE		RSTEP		RDIR		ENBL
--					data <= 	"11" & 	"00" & 	"0" & 	"0010" & "0" & 	"0" & 	"0"; 

				-- TEST: 
--					data <= 	"11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1"; 	-- 
		
		controlRegisterController3(0) <= "11" & 	"00" & 	"0" & 	"0100" & "0" & 	"0" & 	"1";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------		
--		TORQUE
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				--	7-0 	TORQUE 	8 		R/W 	0xFFh 	Sets full-scale output current for both H-bridges
				--	10-8 	SMPLTH 	3 		R/W 	001 		Back EMF sample threshold
				--													000: 50 s
				--													001: 100 s
				--													010: 200 s
				--													011: 300 s
				--													100: 400 s
				--													101: 600 s
				--													110: 800 s
				--													111: 1000 s
				--	11		Reserved	1 		- 		- 			Reserved
				
				-- DEFAULT: 
				-- 		  Reserved	SMPLTH	TORQUE
--					data <= "0" & 		"001" & 	"11111111";
				
				-- TEST: 
--					data <= "0" & 		"001" & 	"00101011";		--

		controlRegisterController3(1) <= "0" & 		"001" & 	"00101011";
-----------------------------------------------------------------------------------------	


-----------------------------------------------------------------------------------------
--		OFF
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	TOFF 		8 		R/W 	0x30h 	Sets fixed off time, in increments of 500 ns
				-- 												0x00h: 500 ns
				-- 												0xFFh: 128 s
				-- 8 		PWMMODE 	1 		R/W 	0 			0: Use internal indexer 
				-- 												1: Bypass indexer, use xINx inputs to control outputs
				--11-9	Reserved	3 		- 		- 			Reserved
				
				-- DEFAULT: 
				-- 		  Reserved	PWMMODE	TOFF
--					data <= "000" & 	"0" & 	"00110000";
				
				-- TEST: 
--					data <= "000" & 	"0" & 	"00001011";		-- 
				
		controlRegisterController3(2) <= "000" & 	"0" & 	"00001011";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--		BLANK
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				--	7-0 	TBLANK 	8 		R/W 	0x80h 	Sets current trip blanking time, in increments of 20 ns
				-- 												(bei den jetzt aufgefhrten Zeiten scheint ein Fehler im Datenblatt zu sein!)
				--													0x00h: 1 s
				-- 												
				-- 												0x32h: 1 s	 
				-- 												0x33h: 1.02 s
				-- 												
				-- 												0xFEh: 5.10 s
				-- 												0xFFh: 5.12 s
				-- 												Also sets minimum on-time of PWM
				-- 8 		ABT 		1 		R/W 	0 			0: Disable adaptive blanking time 1: Enable adaptive blanking time
				-- 11-9 	Reserved	3 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	ABT		TBLANK
--					data <= "000" & 	"0" & 	"10000000";
				
				-- TEST: 
--					data <= "000" & 	"0" & 	"00110001";		-- 
				
		controlRegisterController3(3) <= "000" & 	"0" & 	"00110001";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			DECAY
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	TDECAY 	8 		R/W 	0x10h 	Sets mixed decay transition time, in increments of 500 ns
				-- 10-8 	DECMOD 	3 		R/W 	001 		000: Force slow decay at all times
				--													001: Slow decay for increasing current, mixed decay for
				--														  decreasing current (indexer mode only)
				--													010: Force fast decay at all times
				-- 												011: Use mixed decay at all times
				--													100: Slow decay for increasing current, auto mixed decay for
				--														  decreasing current (indexer mode only)
				--													101: Use auto mixed decay at all times
				--													110 - 111: Reserved
				-- 11 	Reserved	1 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	DECMOD		TDECAY
--					data <= "0" & 		"001" & 		"00010000";
				
				-- TEST: 
--					data <= "0" & 		"101" & 		"00000100";		-- 
				
		controlRegisterController3(4) <= "0" & 		"101" & 		"00000100";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------	
--			STALL 
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 7-0 	SDTHR 	8 		R/W 	0x40h 	Sets stall detect threshold The correct setting needs to be determined experimentally
				-- 9-8 	SDCNT 	2 		R/W 	00 		00: STALLn asserted on first step with back EMF below SDTHR
				-- 												01: STALLn asserted after 2 steps 
				-- 												10: STALLn asserted after 4 steps
				--													11: STALLn asserted after 8 steps
				--11-10 	VDIV 		2 		R/W 	00 		00: Back EMF is divided by 32
				-- 												01: Back EMF is divided by 16 
				--													10: Back EMF is divided by 8
				--													11: Back EMF is divided by 4
				
				-- DEFAULT: 
				--			  VDIV	SDCNT		SDTHR
--					data <= "00" &	"00" & 	"01000000";
				
				-- TEST: 
--					data <= "11" &	"11" & 	"01000000";		-- SDTHR = 64 
				
		controlRegisterController3(5) <= "11" &	"11" & 	"01000000";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			DRIVE
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 1-0 	OCPTH 	2 		R/W 	00 		OCP threshold
				--													00: 250 mV
				-- 												01: 500 mV
				--													10: 750 mV
				--													11: 1000 mV
				-- 3-2 	OCPDEG 	2 		R/W 	01 		OCP deglitch time
				--													00: 1 s
				-- 												01: 2 s
				--													10: 4 s
				--													11: 8 s
				-- 5-4 	TDRIVEN 	2 		R/W 	01 		Low-side gate drive time
				--													00: 250 ns
				-- 												01: 500 ns
				--													10: 1 s
				--													11: 2 s
				-- 7-6 	TDRIVEP 	2 		R/W 	01 		High-side gate drive time
				--													00: 250 ns
				-- 												01: 500 ns
				--													10: 1 s
				--													11: 2 s
				-- 9-8 	IDRIVEN 	2 		R/W 	00			Low-side gate drive peak current
				--													00: 100 mA peak (sink)
				-- 												01: 200 mA peak (sink)
				--													10: 300 mA peak (sink)
				--													11: 400 mA peak (sink)
				-- 11-10	IDRIVEP 	2 		R/W 	00			High-side gate drive peak current
				--													00: 50 mA peak (source)
				-- 												01: 100 mA peak (source)
				--													10: 150 mA peak (source)
				--													11: 200 mA peak (source)
				
				-- DEFAULT: 
				--			  IDRIVEP	IDRIVEN	TDRIVEP	TDRIVEN	OCPDEG	OCPTH
--					data <= "00" &		"00" & 	"01" &	"01" & 	"01" &	"00";
				
				-- TEST: 
--					data <= "01" &		"01" & 	"01" &	"01" & 	"01" &	"00";
				
		controlRegisterController3(6) <= "01" &		"01" & 	"01" &	"01" & 	"01" &	"00";
-----------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------
--			STATUS
				-- BIT 	NAME 		SIZE 	R/W 	DEFAULT 	DESCRIPTION
				-- 0 		OTS 		1 		R 		0 			0: Normal operation
				-- 												1: Device has entered overtemperature shutdown
				--													OTS bit will clear once temperature has fallen to safe levels
				-- 1 		AOCP 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel A overcurrent shutdown
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 2 		BOCP 		1 		R/W 	0			0: Normal operation
				-- 												1: Channel B overcurrent shutdown
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 3 		APDF 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel A predriver fault
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 4 		BPDF 		1 		R/W 	0 			0: Normal operation
				-- 												1: Channel B predriver fault
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 5 		UVLO 		1 		R 		0 			0: Normal operation
				-- 												1: Undervoltage lockout
				--													UVLO bit will clear after VM has increased over VUVLO
				-- 6 		STD 		1 		R 		0 			0: Normal operation 
				-- 												1: Stall detected
				-- 7 		STDLAT 	1 		R/W 	0 			0: Normal operation
				-- 												1: Latched stall detect
				--													Write a '0' to this bit to clear the fault and resume operation
				-- 11-8 	Reserved	4 		- 		- 			Reserved
				
				-- DEFAULT: 
				--			  Reserved	STDLAT	STD	UVLO	BPDF	APDF	BOCP	AOCP	OTS
--					data <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
				
				-- TEST: 
--					data <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
				
		controlRegisterController3(7) <= "0000" &	"0" & 	"0" &	"0" & "0" &	"0" &	"0" & "0" &	"0";
-----------------------------------------------------------------------------------------

----------------------------------------------------------------
----------------------------------------------------------------
--------------------- BIS HIER CONTROLLER 3 --------------------
----------------------------------------------------------------
----------------------------------------------------------------
		end if; 
	end process; 
	
	
end Behavioral;



